unit Tables;
{ প ⠡ 
}

{$A-,B-,F-,I+,R-,S+,V-}

interface

uses Crt,Keystrok,Memory,Graph,Windows,Chars14,Messages;

type
  Places=(Back,Text);
  PlaceColors=array[Places] of byte;

  EntryPtr=^EntryRec;
  EntryRec=record
             obj: pointer;
             name: StringPtr;
             ypos: word;
             pos: byte;
             dec: byte;
             helpid: word;
             next,prec: EntryPtr;
             status: byte;
           end;

  table=^TableRec;
  TableRec=record
             win: window;
             encol,dscol,curcol:PlaceColors;
             namesize,xsize,ysize:word;
             winpos,namepos:word;
             namecol,items: byte;
             item: EntryPtr;
             last: EntryPtr;
             current: EntryPtr;
             status: byte;
           end;

const
  EnablEntry=$80;
  CurEntry=$40;
  StringEntry=0;
  IntEntry=$1;
  ByteEntry=$2;
  WordEntry=$3;
  RealEntry=$4;
  EnumEntry=$10;
  CommEntry=$20;

  Enabled=true;
  Disabled=false;

  Shown=$80;
  Working=$40;

procedure CreateTable(var tbl:table; win:window; namecol:byte;
                        encol,dscol,curcol:PlaceColors);

procedure CreateEntry(var obj; name:string; pos:byte; itemtype:byte;
                          state:boolean; helpid:word);

procedure CreateRealEntry(var obj; name:string; pos,dec:byte;
                          state:boolean; helpid:word);

procedure CreateEnumEntry(var obj; name:string; var desc; num,length:byte;
                          state:boolean; helpid:word);

procedure CreateComment(name:string);

procedure EnableEntry(tbl:table; var obj);

procedure DisableEntry(tbl:table; var obj);

procedure DrawTable(tbl:table);

procedure ModifyTable(tbl:table);

procedure CloseTable(tbl:table);

procedure DestroyTable(tbl:table);

function TableShown(tbl:table):boolean;

var
  CurTable: table;

implementation

type
  EnumRec=record
            obj: pointer;
            desc: pointer;
            num: byte;
          end;

const
  CodeMask=$1f;

{$I BITS.INC}

{*********** auxiliary procedures **************}

procedure SetMax(var acq:word; value:word);
begin
  if value>acq then acq:=value
end;

procedure ConvertToString(var obj; var s:string; pos,dec,code:byte);
type
  buffer=array[0..256] of byte;
var
  p: ^EnumRec;
begin
  case code of
    IntEntry:  s:=ConvertFromInteger(integer(obj),pos);
    ByteEntry: s:=ConvertFromInteger(byte(obj),pos);
    WordEntry: s:=ConvertFromInteger(word(obj),pos);
    RealEntry: s:=ConvertFromReal(real(obj),pos,dec);
    EnumEntry: begin
                 p:=@obj;
                 Move(buffer(p^.desc^)[byte(p^.obj^)*(pos+1)],s,pos+1);
               end;
    else begin
      s:=string(obj); if Length(s)>pos then s[0]:=Chr(pos);
    end;
  end
end;

procedure OutputEntry(tbl:TableRec; p:EntryPtr);
var
  col: PlaceColors;
  s: string;
begin
  if FlagIsSet(p^.status,CommEntry) then Exit;
  with tbl do begin
    if FlagIsSet(p^.status,EnablEntry) then col:=encol else col:=dscol;
    SetColor(col[Text]); SetFillStyle(SolidFill,col[Back]);
    ConvertToString(p^.obj^,s,p^.pos,p^.dec,p^.status and CodeMask);
    Bar(winpos,p^.ypos,winpos+p^.pos*CharWidth+2*TextOffsetX-1,
          p^.ypos+CharHeight+2*TextOffsetY-1);
    OutTextXY(winpos+TextOffsetX,p^.ypos+TextOffsetY,s);
  end
end;

procedure InputEntry(var obj; anscol,pos,dec,vcode:byte; var key:word;
                       helpid:word);
var
  intwrk: integer;
  bytewrk: byte;
  wordwrk: word;
  realwrk: real;
  buf: string;
  code: integer;
begin
  case vcode of
    IntEntry:  begin
                 intwrk:=integer(obj);
                 buf:=ConvertFromInteger(intwrk,pos);
                 repeat
                   code:=0;
                   EnterString(buf,anscol,pos,TextOffsetX,['0'..'9','-'],key,
                                 helpid);
                   Val(buf,intwrk,code);
                   EraseWindow(CurWindow);
                 until code=0;
                 Move(intwrk,obj,sizeof(integer));
               end;
    ByteEntry: begin
                 bytewrk:=byte(obj);
                 buf:=ConvertFromInteger(bytewrk,pos);
                 repeat
                   code:=0;
                   EnterString(buf,anscol,pos,TextOffsetX,['0'..'9'],key,
                                 helpid);
                   Val(buf,bytewrk,code);
                   EraseWindow(CurWindow);
                 until code=0;
                 Move(bytewrk,obj,sizeof(byte));
               end;
    WordEntry: begin
                 wordwrk:=word(obj);
                 buf:=ConvertFromInteger(wordwrk,pos);
                 repeat
                   code:=0;
                   EnterString(buf,anscol,pos,TextOffsetX,['0'..'9'],key,
                                 helpid);
                   Val(buf,wordwrk,code);
                   EraseWindow(CurWindow);
                 until code=0;
                 Move(wordwrk,obj,sizeof(word));
               end;
    RealEntry: begin
                 realwrk:=real(obj);
                 buf:=ConvertFromReal(realwrk,pos,dec);
                 repeat
                   code:=0;
                   EnterString(buf,anscol,pos,TextOffsetX,
                                 ['0'..'9','-','.','e','E'],key,helpid);
                   Val(buf,realwrk,code);
                   EraseWindow(CurWindow);
                 until code=0;
                 Move(realwrk,obj,sizeof(real));
               end;
    EnumEntry: begin
                 EnterEnum(EnumRec(obj).obj^,anscol,EnumRec(obj).desc^,
                             EnumRec(obj).num,pos,TextOffsetX,key,helpid)
               end;
    else begin
      buf:=string(obj);
      EnterString(buf,anscol,pos,TextOffsetX,[Chr($20)..Chr($ff)],key,
                    helpid);
      EraseWindow(CurWindow);
      Move(buf,obj,Length(buf)+1);
    end;
  end;
end;

function SearchEntry(tbl:TableRec; var obj):EntryPtr;
var
  p: EntryPtr;
begin
  p:=tbl.item;
  if p<>nil then begin
    repeat p:=p^.next until (p^.obj=@obj) or (p=tbl.item);
    if p^.obj<>@obj then SearchEntry:=nil else SearchEntry:=p
  end else begin
    SearchEntry:=nil
  end
end;

procedure CreateSomeEntry(var obj; name:string; pos,dec:byte; itemtype:byte;
                          state:boolean; helpid:word);
begin
  with CurTable^ do begin
    if last=nil then begin
      New(last); last^.next:=last; last^.prec:=last;
      item:=last;
    end else begin
      New(last^.next); last^.next^.next:=item; last^.next^.prec:=last;
      item^.prec:=last^.next; last:=last^.next
    end;
    last^.obj:=@obj; last^.name:=AllocateString(name);
    last^.pos:=pos; last^.dec:=dec; last^.helpid:=helpid;
    if state then last^.status:=itemtype+EnablEntry else
      last^.status:=itemtype;
    Inc(items);
    Inc(ysize,CharHeight+TextOffsetY);
    SetMax(namesize,Length(name)*CharWidth);
    SetMax(xsize,namesize+3*TextOffsetX+pos*CharWidth);
  end;
end;

{*********** external procedures **************}

procedure CreateEntry(var obj; name:string; pos:byte; itemtype:byte;
                          state:boolean; helpid:word);
begin
  CreateSomeEntry(obj,name,pos,0,itemtype,state,helpid)
end;

procedure CreateRealEntry(var obj; name:string; pos,dec:byte;
                          state:boolean; helpid:word);
begin
  CreateSomeEntry(obj,name,pos,dec,RealEntry,state,helpid)
end;

procedure CreateEnumEntry(var obj; name:string; var desc; num,length:byte;
                          state:boolean; helpid:word);
var
  p: ^EnumRec;
begin
  New(p); p^.obj:=@obj; p^.num:=num; p^.desc:=@desc;
  CreateSomeEntry(p^,name,length,0,EnumEntry,state,helpid)
end;

procedure CreateComment(name:string);
begin
  with CurTable^ do begin
    if last=nil then begin
      New(last); last^.next:=last; last^.prec:=last;
      item:=last;
    end else begin
      New(last^.next); last^.next^.next:=item; last^.next^.prec:=last;
      item^.prec:=last^.next; last:=last^.next
    end;
    last^.obj:=nil; last^.name:=AllocateString(name);
    last^.status:=CommEntry;
    Inc(items);
    Inc(ysize,CharHeight+2*TextOffsetY);
    SetMax(namesize,Length(name)*CharWidth);
  end;
end;

procedure CreateTable(var tbl:table; win:window; namecol:byte;
                        encol,dscol,curcol:PlaceColors);
begin
  New(tbl); tbl^.item:=nil; tbl^.last:=nil;
  tbl^.win:=win; tbl^.namecol:=namecol;
  tbl^.encol:=encol; tbl^.dscol:=dscol; tbl^.curcol:=curcol;
  tbl^.namesize:=0; tbl^.xsize:=0; tbl^.ysize:=0; tbl^.items:=0;
  tbl^.status:=0;
  CurTable:=tbl;
end;

procedure EnableEntry(tbl:table; var obj);
var
  p: EntryPtr;
begin
  p:=SearchEntry(tbl^,obj);
  if (p<>nil) and not FlagIsSet(p^.status,EnablEntry) then begin
    SetFlag(p^.status,EnablEntry);
    if FlagIsSet(tbl^.status,Shown) then begin
      SelectWindow(tbl^.win); OutputEntry(tbl^,p);
      if tbl^.current=nil then tbl^.current:=p;
    end;
  end;
end;

procedure DisableEntry(tbl:table; var obj);
var
  p: EntryPtr;
begin
  p:=SearchEntry(tbl^,obj);
  if (p<>nil) and FlagIsSet(p^.status,EnablEntry) then begin
    ClearFlag(p^.status,EnablEntry);
    if FlagIsSet(tbl^.status,Shown) then begin
      SelectWindow(tbl^.win); OutputEntry(tbl^,p);
      if tbl^.current=p then begin
        repeat
          tbl^.current:=tbl^.current^.next
        until FlagIsSet(tbl^.current^.status,EnablEntry) or
                        (tbl^.current^.next=p);
        if (tbl^.current^.next=p) then tbl^.current:=nil;
      end
    end;
  end;
end;

procedure DrawTable(tbl:table);
var
  dh,y: word;
  p: EntryPtr;
begin
  with tbl^ do begin
    if FlagIsSet(status,Shown) then begin
      SelectWindow(win); SetWriteMode(NormalPut);
      p:=item;
      if p=nil then begin
        current:=nil; Exit;
      end;
      repeat
        OutputEntry(tbl^,p); p:=p^.next
      until p=item;
    end else begin
      if GetHeight(win)<ysize then
           FatalError('  室  ⨪');
      dh:=(GetHeight(win)-ysize) div (items+1);
      if GetWidth(win)<xsize then
           FatalError('  室  ਧ⠫');
      namepos:=(GetWidth(win)-xsize) div 2;
      winpos:=namepos+namesize+TextOffsetX;
      OpenWindow(win);
      p:=item; if p=nil then Exit;
      y:=dh;
      SetWriteMode(NormalPut);
      repeat
        p^.ypos:=y; SetColor(namecol);
        OutTextXY(namepos,y+TextOffsetY,p^.name^);
        OutputEntry(tbl^,p);
        p:=p^.next; Inc(y,dh+CharHeight+2*TextOffsetY);
      until p=item;
      SetFlag(status,Shown);
      current:=item;
      while (not FlagIsSet(current^.status,EnablEntry))
               and (current^.next<>item)  do begin
        current:=current^.next
      end;
      if not FlagIsSet(current^.status,EnablEntry) then current:=nil;
    end;
  end;
end;

procedure ModifyTable(tbl:table);
var
  col: PlaceColors;
  s: string;
  curwin: window;
  key: word;
begin
  if not FlagIsSet(tbl^.status,Shown) then Exit;
  if tbl^.current=nil then begin
    repeat
      repeat until KeyPressed;
      key:=ReadKey
    until key=EscapeKey;
    Exit
  end;
  AddSpecialKey(UpKey); AddSpecialKey(DownKey);
  with tbl^ do begin
    repeat
      SelectWindow(win);
      MakeSubWindow(curwin, winpos,
                      winpos+current^.pos*CharWidth+2*TextOffsetX-1,
                      current^.ypos, current^.ypos+CharHeight+2*TextOffsetY-1,
                      NoFrame, curcol[Back],Foreground,EraseOn,CloseOff);
      InputEntry(current^.obj^,curcol[Text],current^.pos,current^.dec,
                  current^.status and CodeMask,key,current^.helpid);
      DestroyWindow(curwin);
      case key of
        0,EscapeKey:
                 begin
                   SelectWindow(tbl^.win); SetWriteMode(NormalPut);
                   OutputEntry(tbl^,current);
                   repeat
                     current:=current^.next;
                   until FlagIsSet(current^.status,EnablEntry);
                 end;
        UpKey:   begin
                   SelectWindow(tbl^.win); SetWriteMode(NormalPut);
                   OutputEntry(tbl^,current);
                   repeat
                     current:=current^.prec;
                   until FlagIsSet(current^.status,EnablEntry);
                   key:=0
                 end;
        DownKey: begin
                   SelectWindow(tbl^.win); SetWriteMode(NormalPut);
                   OutputEntry(tbl^,current);
                   repeat
                     current:=current^.next;
                   until FlagIsSet(current^.status,EnablEntry);
                   key:=0
                 end;
       end;
    until key<>0;
  end;
  RemoveSpecialKey(UpKey); RemoveSpecialKey(DownKey);
end;

procedure CloseTable(tbl:table);
begin
  if FlagIsSet(tbl^.status,Shown) then begin
    CloseWindow(tbl^.win); ClearFlag(tbl^.status,Shown)
  end;
end;

procedure DestroyTable(tbl:table);
var
  p,save: EntryPtr;
  q: ^EnumRec;
begin
  if FlagIsSet(tbl^.status,Shown) then CloseTable(tbl);
  p:=tbl^.item;
  while (p<>tbl^.item) do begin
    if FlagIsSet(p^.status,EnumEntry) then begin
      q:=p^.obj; Dispose(q)
    end;
    FreeString(p^.name); save:=p; p:=p^.next; Dispose(save)
  end;
  Dispose(tbl); tbl:=nil
end;

function TableShown(tbl:table):boolean;
begin
  TableShown:=FlagIsSet(tbl^.status,Shown)
end;

begin
end.