unit Windows;
{}

{$A-,B-,F-,I+,R-,S+,V-}

interface

uses Memory,Graph;

{*************** initialization ***************}

var
  GraphDriver,GraphMode: integer;
  ScreenMaxX,ScreenMaxY: word;

{*************** frames *******************}

type
  frame=^FrameRec;
  FrameRec=object
             use: word;
             next: frame;
             constructor Create;
             destructor Destroy; virtual;
             procedure Draw(x1,x2,y1,y2:word); virtual;
             function UpperBorder:word; virtual;
             function LowerBorder:word; virtual;
             function LeftBorder: word; virtual;
             function RightBorder: word; virtual;
           end;

function NoFrame:frame;

function SingleFrame(SetCol:byte):frame;

function FatFrame(SetCol,SetSize:byte):frame;

function DoubleFrame(SetCol,inter:byte):frame;

function DistorsedFrame(SetCol,Xinter,Yinter:byte):frame;

function OldDoubleFrame(SetCol,inter:byte):frame;

function BookFrame(SetCol,SetSize:byte):frame;

function ShadowFrame(SetCol,SetSize:byte):frame;

{************** windows *******************}

type
  window=^WindowRec;
  WindowRec=record
              x1,x2,y1,y2: word;
              xbeg,xend,ybeg,yend: word;
              frm: frame;
              save: memptr;
              bkcol: byte;
              status: byte;
            end;

const
  BackGround=false;
  ForeGround=true;
  EraseOn=true;
  EraseOff=false;
  CloseOn=true;
  CloseOff=false;

procedure CreateWindow(var win:window; x1,x2,y1,y2:word; frm:frame; col:byte;
                         fore,erase,close:boolean);

procedure CreateWindowInt(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);

procedure CreateSubWindow(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);

procedure DestroyWindow(var win:window);

procedure OpenWindow(win:window);

procedure SelectWindow(win:window);

procedure EraseWindow(win:window);

procedure CloseWindow(win:window);

function UpperBorder(win:window):word;

function LowerBorder(win:window):word;

function LeftBorder(win:window):word;

function RightBorder(win:window):word;

function GetWidth(win:window):word;

function GetHeight(win:window):word;

function GetExtWidth(win:window):word;

function GetExtHeight(win:window):word;

function Shown(win:window):boolean;

function Active(win:window):boolean;

function IsForeground(win:window):boolean;

function IsErased(win:window):boolean;

function IsDisappearing(win:window):boolean;

procedure MakeWindow(var win:window; x1,x2,y1,y2:word; frm:frame; col:byte;
                         fore,erase,close:boolean);

procedure MakeWindowInt(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);

procedure MakeSubWindow(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);

procedure ExpandWindow(var win:window; x1,x2,y1,y2:word);

procedure ExpandWindowInt(var win:window; x1,x2,y1,y2:word);

var
  CurWindow,Whole: window;

{****** error report procedure ********}

procedure FatalError(mess:string);

implementation

var
  TurboExitProc: pointer;

type
  ErrorRecord=record
                n: byte;
                mess: string[43];
              end;

const
  Errors: array[1..37] of ErrorRecord =
            ((n:  1;mess:' 맮 '),
             (n:  2;mess:'  䠩'),
             (n:  3;mess:'   㯠'),
             (n:  4;mess:'᫨誮   䠩'),
             (n:  5;mess:' ࠧ襭   䠩'),
             (n:  6;mess:' "" 䠩'),
             (n:  8;mess:' '),
             (n: 12;mess:'  㯠  䠩'),
             (n: 15;mess:'騩 ᪮'),
             (n: 16;mess:'⪠ 㤠 ⥪騩 ⠫'),
             (n: 17;mess:'⪠ २  ᪮'),
             (n:100;mess:'訡 ⥭  ᪠'),
             (n:101;mess:'訡   '),
             (n:102;mess:'䠩  ᢮ '),
             (n:103;mess:'䠩  '),
             (n:104;mess:'䠩    ⥭'),
             (n:105;mess:'䠩    '),
             (n:106;mess:' ଠ ᫠'),
             (n:150;mess:' 饭  '),
             (n:152;mess:'᪮  ⮢'),
             (n:154;mess:'訡 ஫쭮 㬬  ⥭  ᪠'),
             (n:156;mess:'訡 ᪠ ᪮'),
             (n:158;mess:'  ᥪ  ᪥'),
             (n:159;mess:' ਭ  㬠'),
             (n:160;mess:'訡   ன⢮'),
             (n:161;mess:'訡 ⥭ ன⢠'),
             (n:162;mess:'ᡮ '),
             (n:200;mess:'  '),
             (n:201;mess:'室  ࠭ ᨢ'),
             (n:202;mess:'९ ⥪'),
             (n:203;mess:'墠⪠ ⨢ '),
             (n:204;mess:'쭠   㪠⥫'),
             (n:205;mess:'९    . ᫠'),
             (n:206;mess:'祧 浪  . '),
             (n:207;mess:'쭠 ⢥ '),
             (n:208;mess:' ᯥ '),
             (n:209;mess:'訡 ⥭ 䠩 ૥'));

{$F+}
procedure WindowsExitProc;
{$F-}
label unknown;
var
  i: byte;
begin
  if ErrorAddr<>nil then begin
    CloseGraph;
    for i:=1 to 37 do begin
      if ExitCode=Errors[i].n then begin
        Writeln('⠫쭠 訡: '+Errors[i].mess); ErrorAddr:=nil;
        ExitProc:=TurboExitProc; Exit
      end;
      if ExitCode<Errors[i].n then goto unknown
    end;
unknown:
    Writeln(' ⠫쭠 訡'); ErrorAddr:=nil;
  end;
  ExitProc:=TurboExitProc;
end;

{$I BITS.INC}

{*************** error report procedure ***************}

procedure FatalError(mess:string);
begin
  CloseGraph; Writeln(': ',mess); Halt(10)
end;

{*************** frames *******************}

type
  SingleFrameRec=object(FrameRec)
                   col: byte;
                   constructor Create(SetCol: byte);
                   destructor Destroy; virtual;
                   procedure Draw(x1,x2,y1,y2:word); virtual;
                   function UpperBorder:word; virtual;
                   function LowerBorder:word; virtual;
                   function LeftBorder:word; virtual;
                   function RightBorder:word; virtual;
                 end;
  FatFrameRec=object(SingleFrameRec)
                size: byte;
                constructor Create(SetCol,SetSize: byte);
                destructor Destroy; virtual;
                procedure Draw(x1,x2,y1,y2:word); virtual;
                function UpperBorder:word; virtual;
                function LowerBorder:word; virtual;
                function LeftBorder:word; virtual;
                function RightBorder:word; virtual;
              end;
  DoubleFrameRec=object(FatFrameRec)
                   constructor Create(SetCol,SetSize: byte);
                   destructor Destroy; virtual;
                   procedure Draw(x1,x2,y1,y2:word); virtual;
                 end;
  DistorsedFrameRec=object(FatFrameRec)
                      ysize: byte;
                      constructor Create(SetCol,SetXsize,SetYsize: byte);
                      destructor Destroy; virtual;
                      procedure Draw(x1,x2,y1,y2:word); virtual;
                      function UpperBorder:word; virtual;
                      function LowerBorder:word; virtual;
                    end;
  BookFrameRec=object(FatFrameRec)
                 constructor Create(SetCol,SetSize: byte);
                 destructor Destroy; virtual;
                 procedure Draw(x1,x2,y1,y2:word); virtual;
                 function UpperBorder:word; virtual;
                 function LeftBorder:word; virtual;
               end;
  ShadowFrameRec=object(BookFrameRec)
                   constructor Create(SetCol,SetSize: byte);
                   destructor Destroy; virtual;
                   procedure Draw(x1,x2,y1,y2:word); virtual;
                 end;

var
  FrameChain:frame;

constructor FrameRec.Create;
begin
  use:=0; next:=FrameChain;
  FrameChain:=@Self
end;

destructor FrameRec.Destroy;
var
  p: frame;
begin
  if FrameChain<>nil then begin
    if FrameChain=@Self then FrameChain:=next else begin
      p:=FrameChain;
      while (p^.next<>nil) and (p^.next<>@Self) do p:=p^.next;
      if p^.next=@Self then p^.next:=p^.next^.next
    end
  end
end;

procedure FrameRec.Draw(x1,x2,y1,y2:word);
begin
end;

function FrameRec.UpperBorder:word;
begin
  UpperBorder:=0
end;

function FrameRec.LowerBorder:word;
begin
  LowerBorder:=0
end;

function FrameRec.LeftBorder: word;
begin
  LeftBorder:=0
end;

function FrameRec.RightBorder: word;
begin
  RightBorder:=0
end;

function NoFrame:frame;
var
  p: frame;
begin
  p:=FrameChain;
  while (p<>nil) and (typeof(p^)<>typeof(FrameRec)) do p:=p^.next;
  if p=nil then New(p,Create);
  NoFrame:=p
end;

constructor SingleFrameRec.Create(SetCol:byte);
begin
  FrameRec.Create;
  col:=SetCol
end;

destructor SingleFrameRec.Destroy;
begin
  FrameRec.Destroy;
end;

procedure SingleFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SetColor(col);
  MoveTo(x1-1,y1-1); LineTo(x1-1,y2+1); LineTo(x2+1,y2+1);
  LineTo(x2+1,y1-1); LineTo(x1-1,y1-1)
end;

function SingleFrameRec.UpperBorder:word;
begin
  UpperBorder:=1
end;

function SingleFrameRec.LowerBorder:word;
begin
  LowerBorder:=1
end;

function SingleFrameRec.LeftBorder: word;
begin
  LeftBorder:=1
end;

function SingleFrameRec.RightBorder: word;
begin
  RightBorder:=1
end;

function SingleFrame(SetCol:byte):frame;
var
  p: frame;
  sp: ^SingleFrameRec;
begin
  p:=FrameChain; sp:=nil;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(SingleFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if sp^.col=SetCol then p:=nil else p:=p^.next
    end;
  end;
  if sp=nil then New(sp,Create(SetCol));
  SingleFrame:=sp
end;

constructor FatFrameRec.Create(SetCol,SetSize:byte);
begin
  SingleFrameRec.Create(SetCol); size:=SetSize;
end;

destructor FatFrameRec.Destroy;
begin
  SingleFrameRec.Destroy;
end;

procedure FatFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SetFillStyle(SolidFill,col);
  Bar(x1-size,y1-size,x1-1,y2+size);    {left}
  Bar(x2+1,y1-size,x2+size,y2+size);    {right}
  Bar(x1,y1-size,x2,y1-1);              {upper}
  Bar(x1,y2+1,x2,y2+size);              {lower}
end;

function FatFrameRec.UpperBorder:word;
begin
  UpperBorder:=size
end;

function FatFrameRec.LowerBorder:word;
begin
  LowerBorder:=size
end;

function FatFrameRec.LeftBorder: word;
begin
  LeftBorder:=size
end;

function FatFrameRec.RightBorder: word;
begin
  RightBorder:=size
end;

function FatFrame(SetCol,SetSize:byte):frame;
var
  p: frame;
  sp: ^FatFrameRec;
begin
  p:=FrameChain; sp:=nil;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(FatFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if (sp^.col=SetCol) and (sp^.size=SetSize) then p:=nil else p:=p^.next;
    end;
  end;
  if sp=nil then New(sp,Create(SetCol,SetSize));
  FatFrame:=sp
end;

constructor DoubleFrameRec.Create(SetCol,SetSize:byte);
begin
  FatFrameRec.Create(SetCol,SetSize);
end;

destructor DoubleFrameRec.Destroy;
begin
  FatFrameRec.Destroy;
end;

procedure DoubleFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SingleFrameRec.Draw(x1,x2,y1,y2);
  SingleFrameRec.Draw(x1-size+1,x2+size-1,y1-size+1,y2+size-1)
end;

function OldDoubleFrame(SetCol,inter:byte):frame;
var
  p: frame;
  sp: ^DoubleFrameRec;
  s: byte;
begin
  p:=FrameChain; sp:=nil; s:=inter+2;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(DoubleFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if (sp^.col=SetCol) and (sp^.size=s) then p:=nil else p:=p^.next
    end
  end;
  if sp=nil then New(sp,Create(SetCol,s));
  OldDoubleFrame:=sp
end;

constructor DistorsedFrameRec.Create(SetCol,SetXsize,SetYsize:byte);
begin
  FatFrameRec.Create(SetCol,SetXsize); ysize:=SetYsize
end;

destructor DistorsedFrameRec.Destroy;
begin
  FatFrameRec.Destroy;
end;

procedure DistorsedFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SingleFrameRec.Draw(x1,x2,y1,y2);
  SingleFrameRec.Draw(x1-size+1,x2+size-1,y1-ysize+1,y2+ysize-1)
end;

function DistorsedFrameRec.UpperBorder:word;
begin
  UpperBorder:=ysize
end;

function DistorsedFrameRec.LowerBorder:word;
begin
  LowerBorder:=ysize
end;

function DistorsedFrame(SetCol,Xinter,Yinter:byte):frame;
var
  p: frame;
  sp: ^DistorsedFrameRec;
  s,ys: byte;
begin
  p:=FrameChain; sp:=nil; s:=Xinter+2; ys:=Yinter+2;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(DistorsedFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if (sp^.col=SetCol) and (sp^.size=s) and (sp^.ysize=ys) then
                                 p:=nil else p:=p^.next
    end
  end;
  if sp=nil then New(sp,Create(SetCol,s,ys));
  DistorsedFrame:=sp
end;

function DoubleFrame(SetCol,inter:byte):frame;
var
  xasp,yasp: word;
  l: longint;
begin
  GetAspectRatio(xasp,yasp);
  l:=inter;
  DoubleFrame:=DistorsedFrame(SetCol,inter,(l*xasp+(xasp div 2)) div yasp)
end;

constructor BookFrameRec.Create(SetCol,SetSize:byte);
begin
  FatFrameRec.Create(SetCol,SetSize)
end;

destructor BookFrameRec.Destroy;
begin
  FatFrameRec.Destroy;
end;

procedure BookFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SetColor(col);
  MoveTo(x1-1,y2+size); LineTo(x1-1,y1-1); LineTo(x2+size,y1-1);
  SetFillStyle(SolidFill,col);
  Bar(x2+1,y1,x2+size,y2+size);    {right}
  Bar(x1,y2+1,x2,y2+size);         {lower}
end;

function BookFrameRec.UpperBorder:word;
begin
  UpperBorder:=1
end;

function BookFrameRec.LeftBorder: word;
begin
  LeftBorder:=1
end;

function BookFrame(SetCol,SetSize:byte):frame;
var
  p: frame;
  sp: ^BookFrameRec;
begin
  p:=FrameChain; sp:=nil;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(BookFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if (sp^.col=SetCol) and (sp^.size=SetSize) then p:=nil else p:=p^.next;
    end
  end;
  if sp=nil then New(sp,Create(SetCol,SetSize));
  BookFrame:=sp
end;

constructor ShadowFrameRec.Create(SetCol,SetSize: byte);
begin
  BookFrameRec.Create(SetCol,SetSize)
end;

destructor ShadowFrameRec.Destroy;
begin
  BookFrameRec.Destroy
end;

procedure ShadowFrameRec.Draw(x1,x2,y1,y2:word);
begin
  SetColor(col);
  SingleFrameRec.Draw(x1,x2,y1,y2);
  SetFillStyle(SolidFill,col);
  if size>1 then begin
    Bar(x2+2,y1+size,x2+size,y2+size);    {right}
    Bar(x1+size,y2+2,x2+1,y2+size);       {lower}
  end
end;

function ShadowFrame(SetCol,SetSize:byte):frame;
var
  p: frame;
  sp: ^ShadowFrameRec;
begin
  p:=FrameChain; sp:=nil;
  while p<>nil do begin
    while (p<>nil) and (typeof(p^)<>typeof(ShadowFrameRec)) do p:=p^.next;
    sp:=pointer(p);
    if p<>nil then begin
      if (sp^.col=SetCol) and (sp^.size=SetSize) then p:=nil else p:=p^.next;
    end
  end;
  if sp=nil then New(sp,Create(SetCol,SetSize));
  ShadowFrame:=sp
end;

{************** window management procedures ************}

const
  ForeBit=$80;
  EraseBit=$40;
  CloseBit=$20;
  Opened=1;
  Selected=2;

procedure CreateWindow(var win:window; x1,x2,y1,y2:word; frm:frame; col:byte;
                         fore,erase,close:boolean);
var
  b: byte;
begin
  New(win);
  win^.xbeg:=x1; win^.x1:=x1+frm^.LeftBorder;
  win^.xend:=x2; win^.x2:=x2-frm^.RightBorder;
  win^.ybeg:=y1; win^.y1:=y1+frm^.UpperBorder;
  win^.yend:=y2; win^.y2:=y2-frm^.LowerBorder;
  win^.frm:=frm; Inc(frm^.use);
  win^.bkcol:=col;
  b:=0;
  if fore then SetFlag(b,ForeBit);
  if erase then SetFlag(b,EraseBit);
  if close then SetFlag(b,CloseBit);
  win^.status:=b
end;

procedure CreateWindowInt(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);
var
  i:integer;
  b: byte;
begin
  New(win);
  win^.x1:=x1; win^.xbeg:=x1-frm^.LeftBorder;
  win^.x2:=x2; win^.xend:=x2+frm^.RightBorder;
  win^.y1:=y1; win^.ybeg:=y1-frm^.UpperBorder;
  win^.y2:=y2; win^.yend:=y2+frm^.LowerBorder;
  win^.frm:=frm; Inc(frm^.use);
  win^.bkcol:=col;
  b:=0;
  if fore then SetFlag(b,ForeBit);
  if erase then SetFlag(b,EraseBit);
  if close then SetFlag(b,CloseBit);
  win^.status:=b
end;

procedure CreateSubWindow(var win:window; x1,x2,y1,y2:word; frm:frame;
                            col:byte; fore,erase,close:boolean);
begin
  CreateWindow(win,CurWindow^.x1+x1,CurWindow^.x1+x2,CurWindow^.y1+y1,
                 CurWindow^.y1+y2,frm,col,fore,erase,close)
end;

procedure DestroyWindow(var win:window);
var
  p:frame;
begin
  with win^ do begin
    if FlagIsSet(status,Selected) then SelectWindow(Whole);
    if FlagIsSet(status,Opened) then CloseWindow(win);
    p:=frm;
    Dec(p^.use);
    if p^.use=0 then Dispose(p,Destroy);
    Dispose(win)
  end
end;

procedure OpenWindow(win:window);
var
  size: word;
begin
  SelectWindow(Whole);
  with win^ do begin
    if not FlagIsSet(status,Opened) then begin
      if FlagIsSet(status,ForeBit) then begin
        size:=ImageSize(xbeg,ybeg,xend,yend);
        if (size+2<MaxAvail) and (size<>0) then begin
          Allocate(save,size); GetImage(xbeg,ybeg,xend,yend,save^.buf)
        end else FatalError('    । ')
      end;
      if FlagIsSet(status,EraseBit) then begin
        SetWriteMode(NormalPut);
        SetFillStyle(SolidFill,bkcol);
        Bar(xbeg+frm^.LeftBorder,ybeg+frm^.UpperBorder,xend-frm^.RightBorder,
              yend-frm^.LowerBorder);
      end;
      frm^.Draw(x1,x2,y1,y2); SetFlag(status,Opened);
    end;
  end;
  SelectWindow(win)
end;

procedure SelectWindow(win:window);
begin
  with win^ do begin
    if not FlagIsSet(status,Opened) then OpenWindow(win);
    SetViewPort(x1,y1,x2,y2,ClipOn);
    ClearFlag(CurWindow^.status,Selected);
    CurWindow:=win; SetFlag(status,Selected)
  end
end;

procedure EraseWindow(win:window);
var
  cur: window;
begin
  cur:=CurWindow;
  SelectWindow(Whole); SetWriteMode(NormalPut);
  with win^ do begin
    SetFillStyle(SolidFill,bkcol); Bar(x1,y1,x2,y2);
  end;
  SelectWindow(cur)
end;

procedure CloseWindow(win:window);
var
  cur: window;
begin
  with win^ do begin
    if not FlagIsSet(status,Opened) then Exit;
    if FlagIsSet(status,Selected) then SelectWindow(Whole);
    cur:=CurWindow; SelectWindow(Whole);
    if FlagIsSet(status,ForeBit) then begin
      PutImage(xbeg,ybeg,save^.buf,NormalPut);
      Free(save)
    end else if FlagIsSet(status,CloseBit) then begin
      SetFillStyle(SolidFill,Black); SetWriteMode(NormalPut);
      Bar(xbeg,ybeg,xend,yend)
    end;
    ClearFlag(status,Opened);
    SelectWindow(cur)
  end
end;

function UpperBorder(win:window):word;
begin
  UpperBorder:=win^.frm^.UpperBorder
end;

function LowerBorder(win:window):word;
begin
  LowerBorder:=win^.frm^.LowerBorder
end;

function LeftBorder(win:window):word;
begin
  LeftBorder:=win^.frm^.LeftBorder
end;

function RightBorder(win:window):word;
begin
  RightBorder:=win^.frm^.RightBorder
end;

function GetWidth(win:window):word;
begin
  GetWidth:=win^.x2-win^.x1+1
end;

function GetHeight(win:window):word;
begin
  GetHeight:=win^.y2-win^.y1+1
end;

function GetExtWidth(win:window):word;
begin
  GetExtWidth:=win^.xend-win^.xbeg+1
end;

function GetExtHeight(win:window):word;
begin
  GetExtHeight:=win^.yend-win^.ybeg+1
end;

function Shown(win:window):boolean;
begin
  Shown:=FlagIsSet(win^.status,Opened)
end;

function Active(win:window):boolean;
begin
  Active:=FlagIsSet(win^.status,Selected)
end;

function IsForeground(win:window):boolean;
begin
  IsForeground:=FlagIsSet(win^.status,ForeBit)
end;

function IsErased(win:window):boolean;
begin
  IsErased:=FlagIsSet(win^.status,EraseBit)
end;

function IsDisappearing(win:window):boolean;
begin
  IsDisappearing:=FlagIsSet(win^.status,CloseBit or ForeBit)
end;

procedure MakeWindow(var win:window; x1,x2,y1,y2:word; frm:frame; col:byte;
                         fore,erase,close:boolean);
begin
  CreateWindow(win,x1,x2,y1,y2,frm,col,fore,erase,close);
  OpenWindow(win)
end;

procedure MakeWindowInt(var win:window; x1,x2,y1,y2:word; frm:frame;
                          col:byte; fore,erase,close:boolean);
begin
  CreateWindowInt(win,x1,x2,y1,y2,frm,col,fore,erase,close);
  OpenWindow(win)
end;

procedure MakeSubWindow(var win:window; x1,x2,y1,y2:word; frm:frame;
                          col:byte; fore,erase,close:boolean);
begin
  CreateSubWindow(win,x1,x2,y1,y2,frm,col,fore,erase,close);
  OpenWindow(win)
end;

procedure ExpandWindow(var win:window; x1,x2,y1,y2:word);
var
  sec: window;
  stat: byte;
begin
  stat:=win^.status;
  CreateWindow(sec,x1,x2,y1,y2,win^.frm,win^.bkcol,FlagIsSet(stat,ForeBit),
                 FlagIsSet(stat,EraseBit),FlagIsSet(stat,CloseBit));
  DestroyWindow(win);
  if FlagIsSet(stat,Selected) then SelectWindow(sec)
  else if FlagIsSet(stat,Opened) then OpenWindow(sec);
  win:=sec
end;

procedure ExpandWindowInt(var win:window; x1,x2,y1,y2:word);
var
  sec: window;
  stat: byte;
begin
  stat:=win^.status;
  CreateWindowInt(sec,x1,x2,y1,y2,win^.frm,win^.bkcol,FlagIsSet(stat,ForeBit)
                   ,FlagIsSet(stat,EraseBit),FlagIsSet(stat,CloseBit));
  DestroyWindow(win);
  if FlagIsSet(stat,Selected) then SelectWindow(sec)
  else if FlagIsSet(stat,Opened) then OpenWindow(sec);
  win:=sec
end;

var
  work: integer;

begin
  TurboExitProc:=ExitProc;
 { ExitProc:=@WindowsExitProc;}
  if ParamCount=0 then begin
   GraphDriver:=VGA; GraphMode:=VGAHi
  end
  else begin
    if ParamStr(1)='cga' then begin
      GraphDriver:=CGA; GraphMode:=CGAC1;
    end
    else if ParamStr(1)='cgahi' then begin
      GraphDriver:=CGA; GraphMode:=CGAHi;
    end
    else if ParamStr(1)='egalo' then begin
      GraphDriver:=EGA; GraphMode:=EGALo;
    end
    else GraphDriver:=Detect;
  end;
  InitGraph(GraphDriver,GraphMode,'');
  work:=GraphResult;
  if work<>0 then begin
    Writeln(GraphErrorMsg(work)); Halt(1)
  end;
  ScreenMaxX:=GetMaxX; ScreenMaxY:=GetMaxY;
  FrameChain:=nil;
  New(Whole);
  with Whole^ do begin
    x1:=0; y1:=0; xbeg:=0; ybeg:=0;
    x2:=GetMaxX; y2:=GetMaxY; xend:=x2; yend:=y2;
    frm:=NoFrame; frm^.use:=1; bkcol:=0;
    status:=Selected or Opened;
  end;
  CurWindow:=Whole
end.

